# Generated by Django 2.2.19 on 2021-05-05 14:34

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='VAPIDKeyset',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('_private_key', models.BinaryField(max_length=43)),
            ],
        ),
    ]



import json

from django.conf import settings
from django.http.response import HttpResponse, JsonResponse
from django.shortcuts import get_object_or_404, render
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_GET, require_POST
from djangoldp_account.models import LDPUser
from webpush import send_user_notification

from djangoldp_webpushnotification.models import VAPIDKeyset


@require_POST
@csrf_exempt
def send_push(request):
    try:
        body = request.body
        data = json.loads(body)
        if "head" not in data or "body" not in data or "id" not in data:
            return JsonResponse(status=400, data={"message": "Invalid data format"})
    except TypeError:
        return JsonResponse(status=400, data={"message": "Invalid data format"})

    user = get_object_or_404(LDPUser, pk=data["id"])

    vapid_key = VAPIDKeyset.objects.first()
    if not vapid_key:
        return JsonResponse(status=400, data={"message": "No VAPID key configured"})

    settings.WEBPUSH_SETTINGS["VAPID_PUBLIC_KEY"] = vapid_key.public_key
    settings.WEBPUSH_SETTINGS["VAPID_PRIVATE_KEY"] = vapid_key.private_key.decode()

    if "VAPID_ADMIN_EMAIL" not in settings.WEBPUSH_SETTINGS:
        return JsonResponse(
            status=400, data={"message": "No VAPID admin email configured"}
        )

    payload = {"head": data["head"], "body": data["body"]}
    send_user_notification(user=user, payload=payload, ttl=1000)

    return JsonResponse(status=200, data={"message": "Web push successful"})



from django.contrib import admin
from djangoldp.admin import DjangoLDPAdmin

from .models import VAPIDKeyset


class VAPIDKeysetAdmin(DjangoLDPAdmin):
    readonly_fields = ('public_key_view', 'private_key_view')

    def public_key_view(self, obj):
        return obj.public_key

    def private_key_view(self, obj):
        return obj.private_key

    class Meta:
        verbose_name = 'VAPID key-set'

admin.site.register(VAPIDKeyset, VAPIDKeysetAdmin)



from base64 import urlsafe_b64decode, urlsafe_b64encode

from django.db import models
from ecdsa import NIST256p, SigningKey


class VAPIDKeyset(models.Model):
    _private_key = models.BinaryField(max_length=43)
    
    @property
    def private_key(self):
        if isinstance(self._private_key, memoryview):
            return self._private_key.tobytes()
        return self._private_key

    def __str__(self):
        return "public_key:{}... private_key:{}...".format(
            self.public_key[:10], self.private_key[:10]
        )

    @property
    def public_key(self):
        key_str = self.private_key
        padding = len(key_str) % 4
        key_str += b"=" * padding
        key = SigningKey.from_string(
            urlsafe_b64decode(key_str), curve=NIST256p
        ).get_verifying_key()
        return urlsafe_b64encode(b"\x04" + key.to_string()).strip(b"=")



from django.urls import include, path
from django.views.generic import TemplateView

from .views import send_push

urlpatterns = [
    path("send_push", send_push, name="send_webpush"),
    path("webpush/", include("webpush.urls")),
]
